#!/usr/bin/perl
use File::Spec;
use File::Find ();

package Conversion;

use POSIX qw(strftime);
use constant INFINITY => "2147483647";
use if $^O eq 'MSWin32', 'Win32::TieRegistry',qw( KEY_READ REG_MULTI_SZ );
sub KEY_WOW64_64KEY { 0x0100 }

 

sub new
{
    my $class = shift;
    my $self = {
                 record => 0,
        conversion_time => 0,
                 method => '',
    };

    bless $self, $class;
    return $self;
}

 
sub get_bpimagelist {
    if ($^O eq 'MSWin32') { 
        return File::Spec->catfile( get_nbbinpath(),  qw( admincmd bpimagelist.exe ) );
    } else { 
        return File::Spec->catfile( get_nbbinpath(),  qw( admincmd bpimagelist ) );
    }
 
}

sub bpps_cmd {
    my $name = "bpps";
    my $file = $^O eq 'MSWin32' ? $name . ".exe" : $name;
    return File::Spec->catfile( get_nbbinpath(),  $file );
}

sub get_nbbinpath { 
    return File::Spec->catfile( get_installdir(),  qw( netbackup  bin ) );
}

sub get_nbdbpath {
    return File::Spec->catfile( get_installdir(),  qw( netbackup  db ) );
}
 
sub get_nb_db_image_path {
    return File::Spec->catfile( get_nbdbpath(),  qw( images ) );
}
 
sub get_installdir {
 
    my ( $dir, $type) = ();
 
    if ($^O eq 'MSWin32') {
        my $query_key = 'LMachine/SOFTWARE/Veritas/NetBackup/CurrentVersion';
        my $current_version_node = Win32::TieRegistry->Open( $query_key,
            { Access => KEY_READ() | KEY_WOW64_64KEY(), Delimiter => '/' } );

        if ($current_version_node) {
            ( $dir, $type ) = $current_version_node->GetValue('INSTALLDIR');
        }
    } else {
            $dir = '/usr/openv';
    }
 
    return $dir;
}
 

sub set_method {
     my ($self, $method) = @_;
     $self->{method} = $method;
}

sub technote_url {'https://www.veritas.com/content/support/en_US/article.100048600'}

sub description_pre {
    my ( $self ) = @_;

    my $description = sprintf <<'PRE_DESCRIPTION', $self->technote_url();
NetBackup 9.0 and later versions support the
expiration dates that extend beyond the year 2038. To ensure compatibility
with previous NetBackup versions, all items with an infinite expiration date
are updated to reflect the new infinite expiration date value. This
conversion may extend the time that is required to complete the upgrade.
Review the following article for more information:

%s
PRE_DESCRIPTION

    return $description;
}

# Get the path to bpps command
# Use run_sys_cmd to execute bpps
# If execution fails, debug log it and return
# Check bpps for required processes
#   bpcd
#   bpdbm
#   bpinetd - Windows only?
#   dbsrv
#   nbemm
#   vnetd
# If they don't all exist, debug log it and return
# If they are all present, return 'true'
sub required_processes {
    my ( $self ) = @_;

    my $bpps_cmd = bpps_cmd();
    my $bpps_out = `$bpps_cmd`;
    my $status = $?;

    if ($status != 0) {
        print "Unable to execute bpps: [$bpps_out] (error: $status)";
        return 0;

    } else {

        my @missing_processes;
        my @processes = qw( bpcd bpdbm dbsrv nbemm vnetd );
        push @processes, 'bpinetd' if $^O eq 'MSWin32';
        for my $process ( @processes ) {
            if ( $bpps_out !~ m{ $process }xms ) {
                push @missing_processes, $process;
            }
        }
        if ( @missing_processes ) {
            print( 'Required processes missing: [', join( ', ', @missing_processes ), ']' );
            return 0;
        }

        return 1;
    }
}


sub get_infinite_image_count_bpimagelist{

    my ( $self ) = @_;

    my $infinity = INFINITY;
    my $bpimagelist_cmd = get_bpimagelist();

    if ( -e $bpimagelist_cmd) {

        my $bpimagelist_cmd = sprintf( q("%s" -d "01/01/1970"), $bpimagelist_cmd );
        open my $fh, "$bpimagelist_cmd |";
        while ( <$fh> ) {

            # Working on $_, look for line that...
            # Start with IMAGE or FRAG and also contain the number to represent INFINITY
            # The FRAG must be the first fragment of the copy (could be copy 1-10)
            m{
                \A IMAGE .* $infinity |
                \A FRAG \s (?: [0-9] | 10 ) \s 1 \s .* $infinity
            }xms and $self->{record}++;
        }
        print "\t-- OK\n";
        return 1;

    }else{

        print "\t!! ERROR: command bpimagelist doesn't exist [$bpimagelist_cmd]\n";
        print "\t-- Failed\n";
        return 0;
    }
}


sub get_infinite_image_count_find {
    my ( $self ) = @_;

    File::Find::find(
        {
            follow      => 1,
            follow_skip => 2,
            wanted      => sub {
                return unless -f;
                return unless m{
                    \.f \z    |  # Must be .f file or
                    \.f.Z \z  |  # A .f.Z file or
                    \.f.Zl \z |  # A .f.Zl file or
                    \.f.Zs \z    # A .f.Zs file or
                }xms;

                $self->{record}++;
            },
        },
        get_nb_db_image_path()
    );
}



sub format_stats{
    my ( $self ) = @_;

    my $date = strftime('%Y-%m-%d %H:%M', localtime());;
    my $max_process_time = 900;
      my $process_factor = 1000;
           my $time_rate = 60;

    if ($self->{record}/$process_factor > $max_process_time) {
        my $min = $self->{record}/($process_factor*$time_rate);
        my $hour = $min/$time_rate;
        $min %= $time_rate;
        $conversion_time = sprintf("%d:%02d (hh:mm)",$hour, $min);
    } else {
        $conversion_time = strftime('less than %M minutes', gmtime($max_process_time));
    }

    my $nb_state  
        = $self->{method} eq 'bpimagelist' ? 'online'
        : $self->{method} eq 'find'        ? 'offline'
        :                                    'unknown';

    my $result = 
        sprintf <<'END_STATS_MSG', $date, $nb_state, $self->{record}, $conversion_time;
      Date of collection: %s
         NetBackup state: %s
           Records found: %s
Conversion time estimate: %s
END_STATS_MSG

    print $result;
} 
 


sub get_conversion_cal_time {
    
    my ($self) = @_;

    print $self->description_pre();
    print "\nCheck start :\n";

    if ( -e get_nb_db_image_path() ) {

    }else{
        print " Inapplicable as image directory does not exist\n";
        print " [" . get_nb_db_image_path() . "]\n";
        return;
    }


    if ( $self->required_processes() ) {
        print " ---> Use <bpimagelist> to count images need infinite conversion...\n";
        if ($self->get_infinite_image_count_bpimagelist()) {
            # bpimagelist exist 
            $self->set_method( 'bpimagelist' );
        }
    }
       

    if ($self->{method} eq '') {
        print " ---> Use <find> to count all images...\n";
        $self->get_infinite_image_count_find();
        $self->set_method( 'find' );
        print "\t-- OK\n";
    }

    print "\nCheck result: \n";
    $self->format_stats();
}

my $conv = new Conversion;
$conv->get_conversion_cal_time();